# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

import GridUtils
from Game import GameState

class PegJumpGameState(GameState):
    def __init__(self, dimensions=None, params=None, players=2, reset=True):
        GameState.__init__(self, dimensions, params, 1, reset)
        
    def successors(self):
        for c in self.enumerate_coords_with_value(1):
            for ndir in GridUtils.ortho_direction_array(self.grid.ndim):
                # try to jump over existing piece and land in empty cell
                jumppos = GridUtils.add_tuples(c, ndir)
                endpos = GridUtils.add_tuples(jumppos, ndir)
                if self.valid_coords(endpos) and self.grid[jumppos]==1 and self.grid[endpos]==0:
                    nextgrid = self.grid.copy()
                    nextgrid[c] = nextgrid[jumppos] = 0
                    nextgrid[endpos] = 1
                    yield ( (c, endpos), self.next_state_with_grid(nextgrid) )
                    
    def animations_for_move(self, move, newstate):
        import PGPieceAnimation
        # move from start to end, remove middle (which is the average of start and end)
        jump_anim = PGPieceAnimation.move_animation(move[0], move[1])
        jumppos = tuple(((x+y)//2) for x,y in zip(*move))
        remove_anim = PGPieceAnimation.remove_animation(jumppos)
        return PGPieceAnimation.AnimationGroup([jump_anim, remove_anim])

